@@ -48,3 +48,22 @@ func TestURLFormat(t *testing.T) {
 		t.Fatalf(resp)
 	}
 }
+
+func TestURLFormatInSubRouter(t *testing.T) {
+	r := chi.NewRouter()
+
+	r.Route("/articles/{articleID}", func(r chi.Router) {
+		r.Use(URLFormat)
+		r.Get("/subroute", func(w http.ResponseWriter, r *http.Request) {
+			articleID := chi.URLParam(r, "articleID")
+			w.Write([]byte(articleID))
+		})
+	})
+
+	ts := httptest.NewServer(r)
+	defer ts.Close()
+
+	if _, resp := testRequest(t, ts, "GET", "/articles/1/subroute.json", nil); resp != "1" {
+		t.Fatalf(resp)
+	}
+}
